@@ -6,8 +6,6 @@
 
 #include "ISceneNode.h"
 
-namespace irr
-{
 namespace scene
 {
 
@@ -37,4 +35,3 @@ private:
 };
 
 } // end namespace scene
-} // end namespace irr
