@@ -3,15 +3,10 @@
 // Copyright (C) 2025 grorp
 
 #include "l_pause_menu.h"
+#include "client/keycode.h"
 #include "gui/mainmenumanager.h"
 #include "lua_api/l_internal.h"
-
-
-int ModApiPauseMenu::l_show_keys_menu(lua_State *L)
-{
-	g_gamecallback->keyConfig();
-	return 0;
-}
+#include "client/client.h"
 
 
 int ModApiPauseMenu::l_show_touchscreen_layout(lua_State *L)
@@ -21,8 +16,15 @@ int ModApiPauseMenu::l_show_touchscreen_layout(lua_State *L)
 }
 
 
+int ModApiPauseMenu::l_is_internal_server(lua_State *L)
+{
+	lua_pushboolean(L, getClient(L)->m_internal_server);
+	return 1;
+}
+
+
 void ModApiPauseMenu::Initialize(lua_State *L, int top)
 {
-	API_FCT(show_keys_menu);
 	API_FCT(show_touchscreen_layout);
+	API_FCT(is_internal_server);
 }
